@@ -13,11 +13,14 @@ from shutil import copyfile
 ##########################################################################
 ##                          Global constants                            ##
 ##########################################################################
-CLIENT_ID = "844fde0b2950ccf35329"  # The ID of this app for authorization
+# The Thingiverse ID of this app for requesting an API key
+THINGIVERSE_CLIENT_ID = "844fde0b2950ccf35329"  
 
 ##########################################################################
 ##                            Helpers                                   ##
 ##########################################################################
+
+########## General
 def create_textfile(path, data):
     """Creates a new textfile, backing up already existing files if found"""
     # Backup file if it already exists
@@ -30,15 +33,15 @@ def create_textfile(path, data):
     with open(path, "w") as f:
                 f.write(data)
 
-
-def deploy_files(access_path, files, whitelist, thingdata, headers):
+########## Thingiverse
+def thingiverse_deploy_files(access_path, files, whitelist, thingdata, headers):
     """Deploys files.."""
 
     ########## File checks
 
     existing_files = json.loads(
                         requests.get("http://api.thingiverse.com/things/"
-                            + str(thingdata["id"])
+                            + str(thingdata["thingiverse_id"])
                             + access_path, headers=headers).text)
 
 
@@ -121,8 +124,6 @@ def deploy_files(access_path, files, whitelist, thingdata, headers):
     for file in files_to_delete:
         print(file["name"])
 
-    print()
-
     ########## File deletions
 
     for file in files_to_delete:
@@ -132,9 +133,9 @@ def deploy_files(access_path, files, whitelist, thingdata, headers):
         deletion_response = json.loads(
                           requests.delete(
                               "http://api.thingiverse.com/things/"
-                                + str(thingdata["id"])
+                                + str(thingdata["thingiverse_id"])
                                 + access_path + "/"
-                                + str(file["id"]),
+                                + str(file["thingiverse_id"]),
                                 headers=headers).text)
 
         #print(json.dumps(deletion_response, indent=4))
@@ -149,7 +150,7 @@ def deploy_files(access_path, files, whitelist, thingdata, headers):
         params = {"filename":file["name"]}
         upload_creds = json.loads(
                         requests.post("http://api.thingiverse.com/things/"
-                                        + str(thingdata["id"])
+                                        + str(thingdata["thingiverse_id"])
                                         + "/files",
                                         data=json.dumps(params),
                                         headers=headers).text)
@@ -174,15 +175,14 @@ def deploy_files(access_path, files, whitelist, thingdata, headers):
                               headers=headers).text)
 
 
-def set_image_order(imgfiles, thingdata, headers):
+def thingiverse_set_image_order(imgfiles, thingdata, headers):
     """Sets image order of recently uploaded pictures, based on filename"""
 
-    print("")
     print("Ranking images based on file names")
 
     existing_images = json.loads(
                         requests.get("http://api.thingiverse.com/things/"
-                            + str(thingdata["id"])
+                            + str(thingdata["thingiverse_id"])
                             + "/images", headers=headers).text)
 
     # Iterate through uploaded files
@@ -208,21 +208,20 @@ def set_image_order(imgfiles, thingdata, headers):
         # Actually patch image with new rank
         params      = {"rank":remote_image["rank"]}
         img_answer2 = requests.patch("http://api.thingiverse.com/things/" +
-                                    str(thingdata["id"]) +
+                                    str(thingdata["thingiverse_id"]) +
                                     "/images/"+
                                     str(remote_image["id"]),
                                     headers=headers,
                                     data=json.dumps(params))
 
-    print("")
     print("All images ranked")
 
 
-def publish_project(thingdata, headers):
+def thingiverse_publish_project(thingdata, headers):
     """Create publish request"""
     # POST /things/{$id}/publish
     PublishAnswer = requests.post("http://api.thingiverse.com/things/"+
-                                str(thingdata["id"])+
+                                str(thingdata["thingiverse_id"])+
                                 "/publish",
                                 headers=headers)
     
@@ -230,17 +229,18 @@ def publish_project(thingdata, headers):
 
 
 ##########################################################################
-##                      Token generation mode                           ##
+##                       Token request mode                             ##
 ##########################################################################
-def request_token():
+########## Thingiverse
+def thingiverse_request_token():
     """Take app client ID and generate an API token with write acces"""
 
-    print("Running in Api token request mode")
+    print("Running in API token request mode")
 
     # Open up a webbrowser with the authorization URL
     webbrowser.open(url = 
         "https://www.thingiverse.com/login/oauth/authorize?client_id="
-        + CLIENT_ID
+        + THINGIVERSE_CLIENT_ID
         + "&response_type=token",
         new=1,
         autoraise=True);
@@ -267,7 +267,7 @@ def request_token():
             print("Key: ")
             print(new_api_key)
             print()
-            print("Now you can use the --deploy-project option!")
+            print("Using this, you can run '--deploy-project-thingiverse <API_KEY>'!")
         else:
             print("Invalid response URL, api token empty.")
             sys.exit(os.EX_USAGE)
@@ -305,18 +305,18 @@ def create_initial_folder_structure(project_path):
 
     # Create initial thingdata.json
     thingdata = {
-                "id"            :"",
-                "name"          :"Threedeploy Project Name",
-                "creator"       :"YourPlatformNameHere",
-                "is_wip"        :True,
-                "license"       :"gpl",
-                "category"      :"3D Printing",
-                "tags"          : [
-                                "YourTagsHere",
-                                "likeThis"
-                                "Threedeploy",
+                "name"                      :"Threedeploy Project Name",
+                "tags"                      : [
+                                            "YourTagsHere",
+                                            "likeThis"
+                                            "Threedeploy",
                 ],
-                "is_published"  :False
+                "thingiverse_id"            :"",
+                "thingiverse_creator"       :"YourThingiverseNameHere",
+                "thingiverse_is_wip"        :True,
+                "thingiverse_license"       :"gpl",
+                "thingiverse_category"      :"3D Printing",
+                "thingiverse_is_published"  :False
     }
     create_textfile(path = project_path + "/thingdata.json",
                     data =json.dumps(thingdata, indent=4))
@@ -368,22 +368,18 @@ def create_initial_folder_structure(project_path):
 ##########################################################################
 ##                     Project deployment mode                          ##
 ##########################################################################
-def deploy_project(project_path, api_token):
+
+########## General
+def deploy_project(project_path, api_token, destination):
     """Deploy the project using an API token generated by --request-token"""
 
-    print("Deploying project")
-
-    ##########################################################################
-    ##                              Init                                    ##
-    ##########################################################################
-
-    headers = {"Authorization": "Bearer " + api_token}
+    print("Deploying project:")
 
     ##########################################################################
     ##                          File parsing                                ##
     ##########################################################################
 
-    # Thing data
+    ########## Thing data
     datapath = project_path + "/thingdata.json"
     if not os.path.isfile(datapath):
         print("thingdata.json does not exist, have you created your project?")
@@ -392,34 +388,9 @@ def deploy_project(project_path, api_token):
     with open(datapath, "r", encoding="utf-8") as f:
         thingdata = json.load(f)
 
-        # check if thing already exists, if id is provided
-        if thingdata["id"] != "":
-            thing = json.loads(
-                        requests.get("http://api.thingiverse.com/things/" 
-                                    + str(thingdata["id"]), 
-                                    headers=headers).text)
+    print(thingdata["name"])
 
-            # Check if we returned an error
-            if "error" in thing:
-                if thing["error"] == "Unauthorized":
-                    print("Unauthorized, is your API key correct? Exiting")
-                    sys.exit()
-
-            # compare provided name with found creator name as sanity check
-            if thingdata["creator"] == thing["creator"]["name"]:
-                mode = "patch"
-                print("Thing already exists, running in patch mode")
-            else:
-                print("""Thing ID specified in flags.json does not belong to 
-                         creator, exiting""")
-                sys.exit(os.EX_NOPERM)
-
-        else:
-            mode = "create"
-            print("No thing ID provided, running in creation mode")
-        print()
-
-    # Description
+    ########## Description
     descpath = project_path + "/README.md"
     with open(descpath, "r", encoding="utf-8") as f:
         description = f.read()
@@ -427,7 +398,7 @@ def deploy_project(project_path, api_token):
         print(description)
         print("----------------------------------------")
 
-    # model / source files
+    ########## model / source files
     threedpath      = project_path + "/3d"
     sourcepath      = project_path + "/source"
     gcodepath       = project_path + "/gcode"
@@ -462,9 +433,9 @@ def deploy_project(project_path, api_token):
     print("Found model files: ")
     for file in modelfiles:
         print(file["name"])
-    print()
+    print("----------------------------------------")
 
-    # Images
+    ########## Images
     imgpath         = project_path + "/img"
     imgfiles        = []
     for file in os.listdir(imgpath):
@@ -473,17 +444,70 @@ def deploy_project(project_path, api_token):
             file.endswith(".bmp")):
             imgfiles.append({"name":file, 
                              "path":os.path.join(imgpath, file),
-                             "id":0})
+                             "thingiverse_id":0})
 
     print("Found image files: ")
     for file in imgfiles:
         print(file["name"])
-    print()
+    print("----------------------------------------")
 
+    ##########################################################################
+    ##                    Site specific deployment                          ##
+    ##########################################################################
+    if destination == 'thingiverse':
+        print("Deploying to Thingiverse!")
+        deploy_thingiverse(api_token, thingdata, project_path, modelfiles, imgfiles)
 
+    elif destination == 'myminifactory':
+        print('MyMiniFactory deployment not implemented yet, sorry')
+        sys.exit(OS.EX_USAGE)
+
+    elif destination == 'prusaprinters':
+        print('PrusaPrinters deployment not implemented yet, sorry')
+        sys.exit(OS.EX_USAGE)
+
+    elif destination == 'thangs':
+        print('Thangs deployment not implemented yet, sorry')
+        sys.exit(OS.EX_USAGE)
+
+########## Thingiverse
+def deploy_thingiverse(api_token, thingdata, project_path, modelfiles, imgfiles):
     ##########################################################################
     ##                     Thingiverse deployment                           ##
     ##########################################################################
+    
+    headers = {"Authorization": "Bearer " + api_token}
+    
+    # check if thing already exists, if id is provided
+    if thingdata["thingiverse_id"] != "":
+        thing = json.loads(
+                    requests.get("http://api.thingiverse.com/things/" 
+                                + str(thingdata["thingiverse_id"]), 
+                                headers=headers).text)
+
+        # Check if we returned an error
+        if "error" in thing:
+            if thing["error"] == "Unauthorized":
+                print("Unauthorized, is your API key correct? Exiting")
+                sys.exit(os.EX_NOPERM)
+            if thing["error"] == "Not Found":
+                print("Thing ID specified but Thing not found, exiting")
+                sys.exit(os.EX_USAGE)
+
+        # compare provided name with found creator name as sanity check
+        if thingdata["thingiverse_creator"] == thing["creator"]["name"]:
+            mode = "patch"
+            print("Thing already exists, running in patch mode")
+        else:
+            print("""Thing ID specified in flags.json does not belong to 
+                        creator, exiting""")
+            sys.exit(os.EX_NOPERM)
+
+    else:
+        mode = "create"
+        print("No thing ID provided, running in creation mode")
+    print("----------------------------------------")
+
     ########## Thing creation
     # If ID wasn't already found, first create thing
     if mode == "create":
@@ -493,12 +517,13 @@ def deploy_project(project_path, api_token):
 
         # initial file creation
         params = {"name":           thingdata["name"],
-                  "license":        thingdata["license"],
-                  "category":       thingdata["category"],
-                 #"description":    description,
-                 #"instructions":   "None provided",
-                  "is_wip":         thingdata["is_wip"],
+                  "thingiverse_license":        thingdata["thingiverse_license"],
+                  "thingiverse_category":       thingdata["thingiverse_category"],
+                 #"description":    currently broken on Thingiverse,
+                 #"instructions":   currently broken on Thingiverse,
+                  "thingiverse_is_wip":         thingdata["thingiverse_is_wip"],
                   "tags":           thingdata["tags"]}
+
         thing = json.loads(
                         requests.post("http://api.thingiverse.com/things/",
                         headers=headers,
@@ -516,7 +541,7 @@ def deploy_project(project_path, api_token):
             print(new_thing_id)
         
         # Update flags document with newly created ID
-        thingdata["id"] = new_thing_id
+        thingdata["thingiverse_id"] = new_thing_id
         with open(datapath, "w", encoding="utf-8") as f:
             f.write(json.dumps(thingdata, indent=4))
 
@@ -530,19 +555,18 @@ def deploy_project(project_path, api_token):
     # Otherwise, go into patching mode
     elif mode == "patch":
         
-        print()
         print("Patching thing")
 
         params = {"name":           thingdata["name"],
-                  "license":        thingdata["license"],
-                  "category":       thingdata["category"],
+                  "thingiverse_license":        thingdata["thingiverse_license"],
+                  "thingiverse_category":       thingdata["thingiverse_category"],
                  #"description":    description,
                  #"instructions":   "None provided",
-                  "is_wip":         thingdata["is_wip"],
+                  "thingiverse_is_wip":         thingdata["thingiverse_is_wip"],
                   "tags":           thingdata["tags"]}
 
         requests.patch("http://api.thingiverse.com/things/"
-                                    + str(thingdata["id"])
+                                    + str(thingdata["thingiverse_id"])
                                     + "/", headers=headers,
                                     data=json.dumps(params))
 
@@ -552,7 +576,7 @@ def deploy_project(project_path, api_token):
         time.sleep(2) 
 
         thing = json.loads(requests.get("http://api.thingiverse.com/things/"
-                                    + str(thingdata["id"])
+                                    + str(thingdata["thingiverse_id"])
                                     + "/", headers=headers).text)
 
         already_published = thing["is_published"]
@@ -562,7 +586,7 @@ def deploy_project(project_path, api_token):
                 f.write(json.dumps(thing, indent=4))
 
         # Check if valid answer received
-        if thing["id"] == thingdata["id"]:
+        if thing["id"] == thingdata["thingiverse_id"]:
             print("Thing patching succesful")
     
         # Remove InitialCreation file on repeat runs
@@ -570,25 +594,26 @@ def deploy_project(project_path, api_token):
             os.remove(project_path + "/InitialCreation")
             print("InitialCreation file removed")
 
+
     # Model file upload
-    print()
+    print("----------------------------------------")
     print("Deploying model files:")
-    deploy_files("/files", modelfiles, "whitelist", thingdata, headers)
+    thingiverse_deploy_files("/files", modelfiles, "whitelist", thingdata, headers)
 
     # Image upload
-    print()
+    print("----------------------------------------")
     print("Deploying images:")
-    deploy_files("/images", imgfiles, modelfiles, thingdata, headers)
-    set_image_order(imgfiles, thingdata, headers)
+    thingiverse_deploy_files("/images", imgfiles, modelfiles, thingdata, headers)
+    thingiverse_set_image_order(imgfiles, thingdata, headers)
 
     # Publishing
-    print()
+    print("----------------------------------------")
     print("Testing if publishing is required")
-    if thingdata["is_published"] and not thing["is_published"]:
+    if thingdata["thingiverse_is_published"] and not thing["is_published"]:
         print("Publishing thing")
-        publish_project(thingdata, headers)
+        thingiverse_publish_project(thingdata, headers)
 
-    elif not thingdata["is_published"]:
+    elif not thingdata["thingiverse_is_published"]:
         print("Publishing not requested")
 
     elif thing["is_published"]:
@@ -596,33 +621,30 @@ def deploy_project(project_path, api_token):
 
 
     # Output thing URL to artifact and terminal
-    thing_url = "https://thingiverse.com/thing:" + str(thingdata["id"])
-    print()
+    thing_url = "https://thingiverse.com/thing:" + str(thingdata["thingiverse_id"])
+    print("----------------------------------------")
     print("Deploying done! Thing URL: ")
     print(thing_url)
     print("Thing ID:")
-    print(thingdata["id"])
+    print(thingdata["thingiverse_id"])
+    print("----------------------------------------")
 
     with open(project_path + "/ThingURL.txt", "w") as f:
         f.write(thing_url)
 
     with open(project_path + "/ThingID.txt", "w") as f:
-        f.write(str(thingdata["id"]))
+        f.write(str(thingdata["thingiverse_id"]))
 
 
 ##########################################################################
 ##                             main()                                   ##
 ##########################################################################
 def main():
-    ##########################################################################
-    ##                              Intro                                   ##
-    ##########################################################################
-
+    
     print()
     print("----------------------------------------")
     print("----------- Threedeploy start ----------")
     print("----------------------------------------")
-    print()
 
 
     ##########################################################################
@@ -637,7 +659,7 @@ def main():
                         help=
     "Path to project structure")
 
-    # optional flag to create project structure at path
+    # Create folder structure mode
     parser.add_argument("--create-project",
                          action="store_true",
                         help=
@@ -645,23 +667,30 @@ def main():
     "Will backup existing files to *.backup_TIMESTAMP"
     )
 
-    # optional clientid input whith which a token is requested
-    parser.add_argument("--request-token", 
+    # Request API token from Thingiverse
+    parser.add_argument("--request-token-thingiverse", 
                         action="store_true",
                         help=
-    "Requests API token if set")
+    "Requests API token from Thingiverse if set")
 
 
-    # optional token which is used for actual write access
-    parser.add_argument("--deploy-project", metavar="apitoken", type=str, 
+    # Deploy to Thingiverse, using the provided Thingiverse API key
+    parser.add_argument("--deploy-project-thingiverse", 
+                        metavar="thingiverse_apitoken",
+                        type=str, 
                         help=
-    "Input API token for deployment, generated by using --create-token")
+    "Deploy to Thingiverse if set. "
+    "Input Thingiverse API token, generated with --request-token-thingiverse")
 
     
     args = parser.parse_args()
 
 
-    # prioritize between different modes
+    ##########################################################################
+    ##                              Modes                                   ##
+    ##########################################################################
+
+    ########## project creation mode
     if args.create_project:
         # Test path
         if not args.path:
@@ -673,10 +702,16 @@ def main():
 
         create_initial_folder_structure(args.path)
 
-    elif args.request_token:
-        request_token()
+    ########## thingiverse API token request
+    elif args.request_token_thingiverse:
+        thingiverse_request_token()
+
+    ########## project deployment
+    elif args.deploy_project_thingiverse:
+        # or myminifactory
+        # or prusaprinters
+        # or thangs
 
-    elif args.deploy_project:
         # Test path
         if not args.path:
             print("Please provide project path with --path <ProjectPath>")
@@ -685,8 +720,16 @@ def main():
             print("The path specified does not exist, exiting")
             sys.exit(os.EX_USAGE)
 
-        deploy_project(args.path, args.deploy_project)
-        
+        # call deployment function, passing destination input
+        if args.deploy_project_thingiverse:
+            deploy_project(args.path, args.deploy_project_thingiverse, 'thingiverse')
+        # elif myminifactory
+        #    deploy_project(args.path, args.deploy-project-thingiverse, 'myminifactory')
+        # elif prusaprinters
+        #    deploy_project(args.path, args.deploy-project-thingiverse, 'prusaprinters')
+        # elif thangs
+        #    deploy_project(args.path, args.deploy-project-thingiverse, 'thangs')
+
     else:
         print("No mode chosen, use --help to figure out which mode you want")
         sys.exit(os.EX_USAGE)
